from rest_framework import serializers
from .admin import site


def get_list_data(self):
    for field in self.get_list_display():
        pass



class CommonModelListSerializer(serializers.ModelSerializer):
    __str__ = serializers.SerializerMethodField()

    def get___str__(self, obj):
        return str(obj)
        
    class Meta:
        model = None
        fields = '__all__'

class CommonModelSerializer(serializers.ModelSerializer):
    __str__ = serializers.SerializerMethodField()

    def get___str__(self, obj):
        return str(obj)

    class Meta:
        model = None
        fields = '__all__'





from django.http import HttpResponse, JsonResponse
from django.conf import settings


CORS_ALLOW_ORIGIN = getattr(settings, 'CORS_ALLOW_ORIGIN', '*')
CORS_ALLOW_METHODS = getattr(settings, 'CORS_ALLOW_METHODS', ['POST', 'GET', 'PUT', 'DELETE', 'OPTIONS', 'PATCH'])
CORS_ALLOW_HEADERS = getattr(settings, 'CORS_ALLOW_HEADERS', ['content-type', 'authorization'])
CORS_ALLOW_CREDENTIALS = getattr(settings, 'CORS_ALLOW_CREDENTIALS', True)
CORS_EXPOSE_HEADERS = getattr(settings, 'CORS_EXPOSE_HEADERS', ['content-type', 'location'])

class CorsMiddleware:
    def __init__(self, get_response):
        self.get_response = get_response

    def set_headers(self, response):
        response['Access-Control-Allow-Origin'] = CORS_ALLOW_ORIGIN
        response['Access-Control-Allow-Methods'] = ','.join(CORS_ALLOW_METHODS)
        response['Access-Control-Allow-Headers'] = ','.join(CORS_ALLOW_HEADERS)
        response['Access-Control-Allow-Credentials'] = 'true' if CORS_ALLOW_CREDENTIALS else 'false'
        response['Access-Control-Expose-Headers'] = ','.join(CORS_EXPOSE_HEADERS)

        return response

    def __call__(self, request):
        #self.process_request(request)
        #self.process_response(request, self.get_response(request))
        response = self.get_response(request)
        response = self.set_headers(response)
        return response

    def _process_request(self, request):
        if 'HTTP_ACCESS_CONTROL_REQUEST_METHOD' in request.META:
            return self._set_headers(HttpResponse())

    def _process_response(self, request, response):
        return self._set_headers(response)



from django.urls import path, include, re_path
from rest_framework import routers

from django_candy import views

app_name = 'django_candy'


router = routers.SimpleRouter()
router.register(r'api/v1/(?P<app_label>[\w]+)/(?P<model_name>[\w]+)', views.CommonViewSet, basename='candy')

urlpatterns = router.urls


urlpatterns += [
    path('api/v1/init/', views.init_view, name='api'),
    path('api/v1/<app_label>/<model_name>/', views.api_view, name='api'),
    re_path('.*', views.index, name='index'),
]


import copy
from django.contrib import admin
from django.apps import apps
from django.conf import settings
from django.utils.text import capfirst
from django.contrib.admin.utils import label_for_field
from django.urls import reverse


class ModelAdmin:
    list_display = []

    autocomplete_fields = ()
    raw_id_fields = ()
    fields = None
    exclude = None
    fieldsets = None
    filter_vertical = ()
    filter_horizontal = ()
    radio_fields = {}
    prepopulated_fields = {}
    formfield_overrides = {}
    readonly_fields = ()
    ordering = None
    sortable_by = None
    view_on_site = True
    show_full_result_count = True
    form_schema = {}
    menu_label = ''

    def __init__(self, model):
        self.model = model
        self.app = model._meta.app_label

    def get_init_data(self, user):
        data = {
            'name': self.model.__name__.lower(),
            'endpoint': '%s/%s' % (self.model._meta.app_label, self.model.__name__.lower()), # :TODO: reverse url
            'app_label': self.model._meta.app_label,
            'verbose_name': self.model._meta.verbose_name,
            'verbose_name_plural': self.model._meta.verbose_name_plural,
            'list_display': self.get_list_display(),
            'list_display_labels': self.get_list_display_labels(),
            'form_schema': self.form_schema,
            'permissions': [],
        }
        return data

    def get_menu_data(self):
        return {
            'name': self.model.__name__,
            'menu_label': self.menu_label or self.model._meta.verbose_name_plural,
            'app_label': self.model._meta.app_label,
        }

    def get_list_display(self):
        if not len(self.list_display):
            return ['__str__']
        return self.list_display

    def get_list_display_labels(self):
        labels = []

        for item in self.get_list_display():
            labels.append(capfirst(label_for_field(item, self.model, self)))

        return labels



class AdminSite:
    def __init__(self):
        self._registry = {} # {'app_label.model_name': (model_class, model_admin)}
        #self._side_menu = []
        #self._top_menu = []

    def register(self, model, admin_class=None):
        key = '.'.join([model._meta.app_label, model.__name__.lower()])

        if key in self._registry:
            return

        if not admin_class:
            admin_class = ModelAdmin

        model_admin = admin_class(model)
        self._registry[key] = (model, model_admin)

    def get_registry(self):
        return copy.deepcopy(self._registry)

    def get_registered(self, app_label, model_name):
        key = '.'.join([app_label, model_name])
        return self._registry.get(key, (None, None))

    def get_side_menu(self):
        """
            Menu structure:

                data = [
                    {
                        'app': 'app_label', 
                        'models': [
                            {'name': 'model_name', ...}
                        ]
                    }, 
                    {
                        
                    }
                ]
        """
        data = []

        registry = self._registry

        app_indices = {} # {app_label: index}

        for key in registry:
            model, model_admin = registry[key]

            menu_data = model_admin.get_menu_data()

            app_label = menu_data['app_label']
            if app_label not in app_indices:
                app_config = apps.get_app_config(app_label)
                data.append({'app_label': app_label, 'verbose_name': app_config.verbose_name, 'models': []})
                app_indices[app_label] = len(data) - 1

            app_index = app_indices[app_label]

            data[app_index]['models'].append(menu_data)
        return data



    def get_top_menu(self):
        return []

    def get_app_structure(self, user):
        data = {}

        registry = self._registry

        for key in registry:
            model, model_admin = registry[key]

            init_data = model_admin.get_init_data(user)

            app_label = init_data['app_label']

            if app_label not in data:
                app_config = apps.get_app_config(app_label)
                data[app_label] = {
                    'name': app_config.name,
                    'verbose_name': app_config.verbose_name,
                    'models': {}
                }

            data[app_label]['models'][init_data['name']] = init_data
        return data

    def get_site_name(self):
        return getattr(settings, 'CANDY_SITE_NAME', 'Candy')

    def get_base_admin_url(self):
        return reverse('django_candy:index')


site = AdminSite()



from django.shortcuts import render
from django.http import HttpResponse, JsonResponse, Http404
from rest_framework import viewsets
from .admin import site
from .serializers import CommonModelSerializer, CommonModelListSerializer
from .utils import LimitOffsetPagination
from django.conf import settings
from django.urls import reverse


def index(request):
    site_name = site.get_site_name()
    base_admin_url = site.get_base_admin_url()
    base_api_url = base_admin_url + 'api/v1/'
    return render(request, 'django_candy/index.html', {
        'site_name': site_name, 'base_admin_url': base_admin_url,
        'base_api_url': base_api_url
        })


def init_view(request):
    data = {
        'apps': site.get_app_structure(request.user),
        'side_menu': site.get_side_menu(),
        'top_menu': site.get_top_menu(),
        'site_name': site.get_site_name()
    }
    return JsonResponse(data)


class CommonViewSet(viewsets.ModelViewSet):
    #permission_classes = (DefaultModelPermissions,)
    #filter_backends = (filters.DjangoFilterBackend,)
    #filterset_class = ChallanFilterSet
    pagination_class = LimitOffsetPagination

    def set_model(self):
        if hasattr(self, 'model'):
            return

        app_label = self.kwargs['app_label']
        model_name = self.kwargs['model_name']

        model, model_admin = site.get_registered(app_label, model_name)

        if not model:
            raise Http404

        self.model = model
        self.model_admin = model_admin

    def get_queryset(self):
        self.set_model()
        return self.model.objects.all()

    def get_serializer_context(self):
        return {
            'request': self.request, 'action': self.action,
            'model_admin': self.model_admin
        }

    def get_serializer_class(self):
        self.set_model()
        if self.action: # == 'list':
            CommonModelListSerializer.Meta.model = self.model
            CommonModelListSerializer.Meta.model_admin = self.model_admin
            CommonModelSerializer.Meta.fields = self.model_admin.get_list_display()
            return CommonModelListSerializer


        CommonModelSerializer.Meta.model = self.model
        return CommonModelSerializer


def api_list_view(request, name):
    if name == 'init':
        return init_view(request)

    return JsonResponse({'version': '1.0', 'logged_in': False})


def api_view(request, name, pk):
    return JsonResponse({'version': '1.0', 'logged_in': False})